@@ -1,8 +1,6 @@
-/* vim: set expandtab ts=8 sw=4: */
-
-/*  $Id$
+/*  $Id: clipman-dialogs.h 4018 2008-03-04 15:15:08Z mmassonnet $
  *
- *  Copyright © 2005 Nick Schermer <nickschermer@gmail.com>
+ *  Copyright (c) 2006-2007 Nick Schermer <nick@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,13 +20,11 @@
 #ifndef CLIPMAN_DIALOGS_H
 #define CLIPMAN_DIALOGS_H
 
+G_BEGIN_DECLS
+
 void
 clipman_configure (XfcePanelPlugin *plugin, ClipmanPlugin *clipman);
 
-void
-clipman_about (XfcePanelPlugin *plugin);
-
-void
-clipman_question (ClipmanAction *action);
+G_END_DECLS
 
 #endif
