@@ -1,6 +1,7 @@
-/*  $Id: clipman.c 2395 2007-01-17 17:42:53Z nick $
+/*  $Id: clipman.c 4019 2008-03-04 18:14:24Z mmassonnet $
  *
  *  Copyright (c) 2006-2007 Nick Schermer <nick@xfce.org>
+ *                     2008 Mike Massonnet <mmassonnet@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -112,56 +113,34 @@ clipman_check_array_len (ClipmanPlugin *clipman)
 
 
 static gchar *
-clipman_create_title (gchar *txt,
+clipman_create_title (gchar *text,
                       gint   length)
 {
-    gchar *s, *t, *u, *buf;
-    gint   i;
-    
-    /* rough string copy */
-    s = g_strndup (txt, length*8 + 8);
-    
-    if (G_UNLIKELY (!s))
-        return NULL;
+  gchar                *short_text, *tmp = NULL;
+  const gchar          *offset;
 
-    if (!g_utf8_validate (s, -1, NULL))
-    {
-        DBG ("Title is not utf8 complaint, we're going to convert it");
-        
-        u = g_locale_to_utf8 (s, -1, NULL, NULL, NULL);
-        g_free (s);
-        s = u;
-        
-        /* Check the title again */
-        if (!g_utf8_validate (s, -1, NULL))
-        {
-            DBG ("Title is still not utf8 complaint, we going to drop this clip");
-            g_free (s);
-            return NULL;
-        }
-    }
-    
-    /* create string length */
-    buf = g_malloc(length*6); /* max length of utf8 char is 6 */
-    g_utf8_strncpy (buf, s, length);
-    g_free (s);
-    s = buf;
-    
-    /* remove tabs and newlines */
-    /* this works when the string is utf8-valid */
-    i = 0;
-    while (s[i] != '\0')
-    {
-        if (s[i] == '\n' || s[i] == '\r' || s[i] == '\t')
-            s[i] = ' ';
-        i++;
-    }
-    
-    /* remove html characters */
-    t = g_markup_escape_text (s, -1);
-    g_free (s);
+  g_return_val_if_fail (G_LIKELY (NULL != text), NULL);
+  g_return_val_if_fail (G_LIKELY (g_utf8_validate (text, -1, NULL)), NULL);
 
-    return t;
+  short_text = g_strstrip (g_strdup (text));
+
+  /* Shorten */
+  if (g_utf8_strlen (short_text, -1) > length)
+    {
+      offset = g_utf8_offset_to_pointer (short_text, length);
+      tmp = g_strndup (short_text, offset - short_text);
+      g_free (short_text);
+
+      short_text = g_strconcat (tmp, "...", NULL); /* Ellipsis */
+      g_free (tmp);
+    }
+
+  /* Cleanup */
+  tmp = g_strdelimit (short_text, "\n\r\t", ' ');
+  short_text = g_markup_escape_text (tmp, -1);
+  g_free (tmp);
+
+  return short_text;
 }
 
 void
@@ -334,30 +313,31 @@ clipman_create_menuitem (ClipmanAction *action,
                          gboolean       bold)
 {
     GtkWidget *mi;
-    GtkLabel  *label;
-    gchar     *title;
-
-    mi = gtk_menu_item_new_with_label  ("");
-
-    if (bold)
-        title = g_strdup_printf("<b>%s</b>", action->clip->title);
-    else
-	title = g_strdup_printf("%s", action->clip->title);
-
+    gchar     *title, *string_num;
 
     if (action->clipman->ItemNumbers)
     {
         if (number < 10)
-            title = g_strdup_printf("<tt><span size=\"smaller\">%d. </span></tt> %s", number ,title);
+            string_num = g_strdup_printf("<tt><span size=\"smaller\">%d. </span></tt> ", number);
         else
-            title = g_strdup_printf("<tt><span size=\"smaller\">%d.</span></tt> %s",  number ,title);
+            string_num = g_strdup_printf("<tt><span size=\"smaller\">%d.</span></tt> ", number);
+    }
+    else
+    {
+        string_num = g_strdup ("");
     }
 
-    label = GTK_LABEL(GTK_BIN(mi)->child);
-    gtk_label_set_markup (label, title);
-    gtk_label_set_single_line_mode (label, TRUE);
-    gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_END);
-    gtk_label_set_max_width_chars (label, width);
+    if (bold)
+        title = g_strdup_printf("%s<b>%s</b>", string_num, action->clip->title);
+    else
+        title = g_strdup_printf("%s%s", string_num, action->clip->title);
+
+    g_free (string_num);
+
+
+    mi = gtk_menu_item_new_with_label ("");
+    gtk_label_set_markup (GTK_LABEL (GTK_BIN (mi)->child), title);
+    g_free (title);
 
     return mi;
 }
